#include <Wire.h>
#include <MPU6050.h>
#include <Sodaq_BMP085.h> // Pressure sensors BPM180
#include <SoftwareSerial.h>
#include <TinyGPS.h>
#include <SPI.h>
#include <SD.h>

File myFile1, myFile2;

long lastTime1, lastTime2;
float flat, flon, falt;

MPU6050 mpu;
Sodaq_BMP085 bmp;

SoftwareSerial HC12(9, 8); // HC-12 TX Pin, HC-12 RX Pin
TinyGPS gps;
SoftwareSerial ss(6, 7); // RX, TX

void setup() 
{
  Serial.begin(9600);

  HC12.begin(9600);               // Serial port to HC12
 
  Serial.println("Initialize MPU6050");

  while(!mpu.begin(MPU6050_SCALE_2000DPS, MPU6050_RANGE_2G))
  {
    Serial.println("Could not find a valid MPU6050 sensor, check wiring!");
    delay(500);
  }

  Serial.println("Initialize BMP180");
  bmp.begin(BMP085_ULTRALOWPOWER);

  Serial.println("Initialize GP-20U7");
  ss.begin(9600);
  smartdelay(1000);

  Serial.print("Initializing SD card...");
  if (!SD.begin(10)) {
    Serial.println("initialization failed!");
    while (1);
  }
  Serial.println("initialization done.");

  myFile1 = SD.open("IMU.txt", FILE_WRITE);
  myFile2 = SD.open("GPS.txt", FILE_WRITE);
  
  lastTime1 = 0;
  lastTime2 = 0;
  
}

void loop()
{

  Serial.println(millis() - lastTime1);
    lastTime1 = millis();

    /* reading sensori */
    Vector normAccel = mpu.readNormalizeAccel();
    Vector normGyro = mpu.readNormalizeGyro();

    //float temp1 = mpu.readTemperature();
    //float temp2 = bmp.readTemperature();

    float p = bmp.readPressure();

    HC12.print("A ");
    HC12.print(lastTime1);
    HC12.print(" ");
    HC12.print(normAccel.XAxis, 4);
    HC12.print(" ");
    HC12.print(normAccel.YAxis, 4);
    HC12.print(" ");
    HC12.print(normAccel.ZAxis, 4);
    HC12.print(" ");
    HC12.print(normGyro.XAxis, 4);
    HC12.print(" ");
    HC12.print(normGyro.YAxis, 4);
    HC12.print(" ");
    HC12.print(normGyro.ZAxis, 4);
    HC12.print(" ");
    /*HC12.print(temp1, 0);
    HC12.print(" ");
    HC12.print(temp2, 0);
    HC12.print(" ");*/
    HC12.println(p, 0);

    /* write on file */
    //myFile1 = SD.open("IMU.txt", FILE_WRITE);
    myFile1.print(lastTime1);
    myFile1.print(" ");
    myFile1.print(normAccel.XAxis, 4);
    myFile1.print(" ");
    myFile1.print(normAccel.YAxis, 4);
    myFile1.print(" ");
    myFile1.print(normAccel.ZAxis, 4);
    myFile1.print(" ");
    myFile1.print(normGyro.XAxis, 4);
    myFile1.print(" ");
    myFile1.print(normGyro.YAxis, 4);
    myFile1.print(" ");
    myFile1.print(normGyro.ZAxis, 4);
    myFile1.print(" ");
    /*myFile1.print(temp1, 0);
    myFile1.print(" ");
    myFile1.print(temp2, 0);
    myFile1.print(" ");*/
    myFile1.println(p, 0);
    //myFile1.close();

    /* sending data */

  if(millis() - lastTime2 > 800) {
  
      lastTime2 = millis();

      smartdelay(0);
      
      gps.f_get_position(&flat, &flon);
      falt = gps.f_altitude();  
  
      HC12.print("B ");
      HC12.print(lastTime2);
      HC12.print(" ");
      HC12.print(flat, 4);
      HC12.print(" ");
      HC12.print(flon, 4);
      HC12.print(" ");
      HC12.println(falt, 4);

      /* write on file */
      //myFile = SD.open("GPS.txt", FILE_WRITE);
      myFile2.print(lastTime2);
      myFile2.print(" ");
      myFile2.print(flat, 4);
      myFile2.print(" ");
      myFile2.print(flon, 4);
      myFile2.print(" ");
      myFile2.print(falt, 4);
      //myFile2.close();
      
   }
  
}
static void smartdelay(unsigned long ms)
{
  unsigned long start = millis();
  do 
  {
    while (ss.available())
      gps.encode(ss.read());
  } while (millis() - start < ms);
}
