#include <Wire.h>
#include <MPU6050.h>
#include <Sodaq_BMP085.h> // Pressure sensors BPM180
#include <SoftwareSerial.h>
#include <TinyGPS.h>
#include <SPI.h>
#include <SD.h>

File myFile;

long lastTime1, lastTime2;
Vector normAccel, normGyro;
float temp, p;
float flat, flon, falt;

String  s;

MPU6050 mpu;
Sodaq_BMP085 bmp;
TinyGPS gps;

SoftwareSerial HC12(9, 8); // HC-12 TX Pin, HC-12 RX Pin
SoftwareSerial ss(6, 7); // RX, TX

char tmp_buffer[8];

void setup() {
  
  Serial.begin(9600);

  HC12.begin(9600);               // Serial port to HC12
 
  Serial.println("Initialize MPU6050");
  while(!mpu.begin(MPU6050_SCALE_2000DPS, MPU6050_RANGE_2G))
  {
    Serial.println("Could not find a valid MPU6050 sensor, check wiring!");
    delay(500);
  }

  Serial.println("Initialize BMP180");
  bmp.begin(BMP085_ULTRALOWPOWER);

  Serial.println("Initialize GP-20U7");
  ss.begin(9600);
  smartdelay(1000);

  /*Serial.print("Initializing SD card...");
  if (!SD.begin(10)) {
    Serial.println("initialization failed!");
    while (1);
  }
  Serial.println("initialization done.");

  myFile = SD.open("output.txt", FILE_WRITE);*/
  
  lastTime1 = 0;
  lastTime2 = 0;

}

void loop() {

  Serial.println(millis() - lastTime1);
  lastTime1 = millis();
  
  normAccel = mpu.readRawAccel();
  normGyro = mpu.readRawGyro();

  temp = bmp.readRawTemperature();
  p = bmp.readRawPressure();
  
  s =  "A " + String(lastTime1);

  //Serial.print(s);
  HC12.print(s);
  myFile.print(s);

  s =  " "  + String(normAccel.XAxis, 0)
     + " "  + String(normAccel.YAxis, 0)
     + " "  + String(normAccel.ZAxis, 0);

  //Serial.print(s);
  HC12.print(s);
  myFile.print(s);
  
  s = " " + String(normGyro.XAxis, 0)
    + " " + String(normGyro.YAxis, 0)
    + " " + String(normGyro.ZAxis, 0);
           
  //Serial.print(s);
  HC12.print(s);
  myFile.print(s);

  s = " " + String(temp,0) + " " + String(p,0);
  
  //Serial.println(s);
  HC12.println(s);
  myFile.println(s);

  /*if(millis() - lastTime2 > 800) {
  
    lastTime2 = millis();

    smartdelay(0);
      
    gps.f_get_position(&flat, &flon);
    falt = gps.f_altitude();

    String s = "B "
             + (String) lastTime2 + " "
             + (String) dtostrf(flat, 7, 4, tmp_buffer) + " "
             + (String) dtostrf(flon, 7, 4, tmp_buffer) + " "
             + (String) dtostrf(falt, 7, 4, tmp_buffer);

    HC12.println(s);
    myFile.println(s);
      
  }*/

}

static void smartdelay(unsigned long ms)
{
  unsigned long start = millis();
  do 
  {
    while (ss.available())
      gps.encode(ss.read());
  } while (millis() - start < ms);
}
